//////////////////////////////////////////////////////////////////////
// aft_t1e1_cfg.cpp - Device Manager Property Page for AFT T1/E1 adapters.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

#define front_end_cfg wandev_conf.fe_cfg

////////////////////////////////////////////////////////////////////////////////
HINSTANCE hInst;

static TCHAR friendly_dev_name[TMP_BUFFER_LEN];

UCHAR released_by_OK_button = 0;

int  iCardType=0;
int	 iCardSubType=0;

unsigned int iDriverMode=0;
unsigned int iAutoWanpipeNumber = 1;

////////////////////////////////////////////////////////////////////////////////

char * framing_options_T1[] = {	"ESF",		//1
								"D4",		//2
								"ESF Japan", //3
								NULL
};

char * framing_options_E1[] = {	"CRC4",		//4
								"non-CRC4",	//5
								"Unframed", //6
								NULL
};

char * t1_lbo_options[] = {	"0dB",		//1
							"7.5dB",	//2
							"15dB",		//3
							"22.5dB",	//4
							"0-110ft",	//5
							"110-220ft",//6
							"220-330ft",//7
							"330-440ft",//8
							"440-550ft",//9
							"550-660ft",	//10
							NULL
};

char * e1_lbo_options[] = {	"12dB",		//1
							"7.5dB",	//2
							NULL
};

/* For E1 only */
//#define WAN_E1_120	0x0B
//#define WAN_E1_75		0x0C

#define AMI_STR		"AMI"
#define B8ZS_STR	"B8ZS"
#define HDB3_STR	"HDB3"

char * line_decoding_T1[] =	{	AMI_STR, //1
								B8ZS_STR //2
};

char * line_decoding_E1[] =	{	AMI_STR, //1
								HDB3_STR //3
};

char * clocking_mode[]	=	{	"Normal",	//1
								"Master"	//2
};

char * clock_reference_port[] =	{"Not Used", "1", "2", "3", "4", "5", "6", "7", "8"};

char * e1_signalling[] = { "CCS", "CAS", NULL };

char *high_impedance_modes[] = {"Off", "On", NULL};

/*
TE_RX_SLEVEL:  Receiver Sensitivity (Max Cable Loss Allowed) (dB)
The user will have the following option to choose:
1. Hi-Imedance mode:
30 dB
22.5 dB
17.5 dB
12 dB

Depending on selection, write the following values into configuration files:
300
225
175
120

2. Normal mode:
12 dB
18 dB
30 dB
36 dB (for T1) 
43 dB (for E1)

Depending on selection, write the following values into configuration files:
120
180
300
360
430
*/

const look_up_t high_impedance_rx_slevel[] = {
	{"30 dB",	WAN_TE1_RX_SLEVEL_30_DB},
	{"22.5 dB",	WAN_TE1_RX_SLEVEL_225_DB},
	{"17.5 dB",	WAN_TE1_RX_SLEVEL_175_DB},
	{"12 dB",	WAN_TE1_RX_SLEVEL_12_DB},
	{NULL,		0}
};

const look_up_t t1_normal_mode_rx_slevel[] = {
	{"12 dB",	WAN_TE1_RX_SLEVEL_12_DB},
	{"18 dB",	WAN_TE1_RX_SLEVEL_18_DB},
	{"30 dB",	WAN_TE1_RX_SLEVEL_30_DB},
	{"36 dB",	WAN_TE1_RX_SLEVEL_36_DB},
	{NULL,		0}
};

const look_up_t e1_normal_mode_rx_slevel[] = {
	{"12 dB",	WAN_TE1_RX_SLEVEL_12_DB},
	{"18 dB",	WAN_TE1_RX_SLEVEL_18_DB},
	{"30 dB",	WAN_TE1_RX_SLEVEL_30_DB},
	{"43 dB",	WAN_TE1_RX_SLEVEL_43_DB},
	{NULL,		0}
};

const look_up_t ignore_yellow_alarm_mode[] = {
	{"No (default)",	WANOPT_NO},
	{"Yes",				WANOPT_YES},
	{NULL,				0}
};

////////////////////////////////////////////////////////////////////////////////
//global configuration variables
static char tmp_registry_buffer[TMP_BUFFER_LEN];

////////////////////////////////////////////////////////////////////////////////
//media functions prototypes
static void ReadConfigurationFromRegistry();

static BOOL SaveT1E1CfgData(HWND hdlg);
int T1E1CfgOnNotify(
	HWND		ParentHwnd,
	LPNMHDR		NmHdr,
	SETUPSTUFF* Params
	);

static void MediaInit(HKEY	hkey, HWND hdlg);

int active_channels_str_invalid_characters_check(char* active_ch_str);
unsigned long get_active_channels(int channel_flag, int start_channel,
                                  int stop_channel,  unsigned char media_type);
int check_channels(int channel_flag, unsigned int start_channel,
                   unsigned int stop_channel, unsigned char media_type, unsigned char is_unframed_e1);

static void disable_enable_T1_controls(HWND hdlg, BOOL flag);
static void disable_enable_E1_controls(HWND hdlg, BOOL flag);
static void disable_enable_T1E1_controls(HWND hdlg, BOOL flag);
////////////////////////////////////////////////////////////////////////////////
void init_te_rx_slevel_combobox(HWND hdlg);
int get_te_rx_slevel_value(HWND hdlg);
////////////////////////////////////////////////////////////////////////////////

SETUPSTUFF * stuff;

PROPSHEETPAGE	aft_firmware_update_page;
///////////////////////////////////////////////////////////////////////////////

BOOL APIENTRY DllMain(HANDLE hModule, DWORD reason, LPVOID lpReserved)
{							// DllMain
	hInst = (HINSTANCE) hModule;
    return TRUE;
}							// DllMain

///////////////////////////////////////////////////////////////////////////////

extern "C" BOOL CALLBACK EnumPropPages(PSP_PROPSHEETPAGE_REQUEST p,
															 LPFNADDPROPSHEETPAGE AddPage,
															 LPARAM lParam)
{	
	HPROPSHEETPAGE	hpage;
	PROPSHEETPAGE	media_page;
	PROPSHEETPAGE	drv_page;
	PROPSHEETPAGE	aft_te1_page;
	PROPSHEETPAGE	aft_analog_page;
	PROPSHEETPAGE	aft_serial_page;
	PROPSHEETPAGE	s518_adsl_page;
	PROPSHEETPAGE	aft_bri_page;
	int				i;
		
	Debug(DBG_SAM_CLASS, "EnumPropPages()\n");

	// Create an auxiliary data structure to pass important information
	// to our WM_INITDIALOG handler. Arrange for a callback when the page
	// is destroyed (the only event we actually care about) to delete this
	// structure
	stuff = new SETUPSTUFF;
	stuff->info = p->DeviceInfoSet;
	stuff->did = p->DeviceInfoData;

	//init Globals
	memset(&Globals_Old_Cfg, 0x00, sizeof(GLOBALS));
	memset(&Globals_New_Cfg, 0x00, sizeof(GLOBALS));

	//initialize channelized cfg 'Old'
	for(i = 0; i < NUM_OF_E1_TIMESLOTS; i++){
		strcpy(Globals_Old_Cfg.aft_logic_ch_cfg[i].active_ch_str,
			NOT_INIT_STR);
		strcpy(Globals_Old_Cfg.aft_logic_ch_cfg[i].line_mode,
			"HDLC");
		strcpy(Globals_Old_Cfg.aft_logic_ch_cfg[i].mtu,
			"1600");
		strcpy(Globals_Old_Cfg.aft_logic_ch_cfg[i].operational_mode,
			"API");
	}

	hkey = SetupDiOpenDevRegKey(stuff->info,
								stuff->did, DICS_FLAG_GLOBAL, 0,
								DIREG_DRV, KEY_READ | KEY_SET_VALUE );

	if (hkey != INVALID_HANDLE_VALUE)
	{	okay = TRUE;
//
//NOTE: not used anymore, WANPIPE "friendly_dev_name" is used instead. Read only as a test.
//
		// Determine the friendly name *or* description of the device
		if (!SetupDiGetDeviceRegistryProperty(	stuff->info, stuff->did,
												SPDRP_FRIENDLYNAME, NULL,
												(PBYTE)friendly_dev_name, sizeof(friendly_dev_name), NULL)	&&
			!SetupDiGetDeviceRegistryProperty(	stuff->info, stuff->did, 
												SPDRP_DEVICEDESC, NULL,
												(PBYTE)friendly_dev_name, sizeof(friendly_dev_name), NULL))
		{
			friendly_dev_name[0] = 0;
		}
	}else
	{	hkey = 0;
		delete stuff;
		return FALSE;
	}

	if (okay == TRUE)
	{	//read the configuration from the registry
		ReadConfigurationFromRegistry();
		
		//Update new cfg with the current (old) cfg, so it will
		//make sense to compare them later.
		memcpy(&Globals_New_Cfg, &Globals_Old_Cfg, sizeof(Globals_Old_Cfg));
	}

	if(iDriverMode == DRV_MODE_AFTUP){

		//aft firmware update page
		memset(&aft_firmware_update_page, 0, sizeof(PROPSHEETPAGE));
		aft_firmware_update_page.dwSize = sizeof(PROPSHEETPAGE);
		aft_firmware_update_page.hInstance = hInst;
		aft_firmware_update_page.pszTemplate = MAKEINTRESOURCE(AFT_FIRMWARE_UPDATE);
		aft_firmware_update_page.pfnDlgProc = AFTFirmwareUpdateDlgProc;
		
		aft_firmware_update_page.lParam = (LPARAM) stuff;
		aft_firmware_update_page.pfnCallback = AFTFirmwareUpdatePageCallbackProc;
		aft_firmware_update_page.dwFlags = PSP_USECALLBACK;

		hpage = CreatePropertySheetPage(&aft_firmware_update_page);
		if (!(*AddPage)(hpage, lParam)){
			DestroyPropertySheetPage(hpage);
		}

	}else{

		switch(iCardType)
		{
		case SDLA_AFT:
			switch(iCardSubType)
			{
			//t1/e1 cards
			case A101_1TE1_SUBSYS_VENDOR:
			case AFT_1TE1_SHARK_SUBSYS_VENDOR:
			case A101_2TE1_SUBSYS_VENDOR:
			case AFT_2TE1_SHARK_SUBSYS_VENDOR:
			case A104_4TE1_SUBSYS_VENDOR:
			case AFT_4TE1_SHARK_SUBSYS_VENDOR:
			case AFT_8TE1_SHARK_SUBSYS_VENDOR:
				//media page
				memset(&media_page, 0, sizeof(PROPSHEETPAGE));
				media_page.dwSize = sizeof(PROPSHEETPAGE);
				media_page.hInstance = hInst;
				media_page.pszTemplate = MAKEINTRESOURCE(MEDIA_CFG_PAGE);
				media_page.pfnDlgProc = AFT_T1E1_PageDlgProc;
				
				media_page.lParam = (LPARAM) stuff;
				media_page.pfnCallback = AFT_T1E1_PageCallbackProc;
				media_page.dwFlags = PSP_USECALLBACK;

				//AFT TE1 logic channels page
				memset(&aft_te1_page, 0, sizeof(PROPSHEETPAGE));
				aft_te1_page.dwSize = sizeof(PROPSHEETPAGE);
				aft_te1_page.hInstance = hInst;
				aft_te1_page.pszTemplate = MAKEINTRESOURCE(AFT_LOGIC_CHANNELS);
				aft_te1_page.pfnDlgProc = AFT_PageDlgProc;
				
				aft_te1_page.lParam = (LPARAM) stuff;
				aft_te1_page.pfnCallback = AFT_PageCallbackProc;
				aft_te1_page.dwFlags = PSP_USECALLBACK;

				// Create a property page and add it to the device manager's property sheet
				//
				//media page
				//
				//This page is added first so it will receive PSPCB_RELEASE (destroy)
				//notification last. So, there we will have to compare the old and new
				//configuration and restart "Virtual adapter" if cfg changed.
				//Also, there will be all the global clean-up.

				//media page
				hpage = CreatePropertySheetPage(&media_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}

				//AFT page
				hpage = CreatePropertySheetPage(&aft_te1_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}
				break;

			case AFT_56K_SHARK_SUBSYS_VENDOR:		//0xA056
				//For 56k card user can choose to run in API/STACK mode for
				//the single channel it has.
				memset(&aft_te1_page, 0, sizeof(PROPSHEETPAGE));
				aft_te1_page.dwSize = sizeof(PROPSHEETPAGE);
				aft_te1_page.hInstance = hInst;
				aft_te1_page.pszTemplate = MAKEINTRESOURCE(AFT_LOGIC_CHANNELS);
				aft_te1_page.pfnDlgProc = AFT_PageDlgProc;
				
				aft_te1_page.lParam = (LPARAM) stuff;
				aft_te1_page.pfnCallback = AFT_T1E1_PageCallbackProc;//<-- note the callback function. It is NOT AFT_PageCallbackProc()!
																	//It will check for config change and ask user for driver restart.
				aft_te1_page.dwFlags = PSP_USECALLBACK;

				//AFT page
				hpage = CreatePropertySheetPage(&aft_te1_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}
				break;

			//all Analog cards
			case A200_REMORA_SHARK_SUBSYS_VENDOR://the Analog port on Flex BRI card also appears as A200
			case A400_REMORA_SHARK_SUBSYS_VENDOR:
			case AFT_A600_SUBSYS_VENDOR:
				//AFT Analog logic channels page
				memset(&aft_analog_page, 0, sizeof(PROPSHEETPAGE));
				aft_analog_page.dwSize = sizeof(PROPSHEETPAGE);
				aft_analog_page.hInstance = hInst;
				aft_analog_page.pszTemplate = MAKEINTRESOURCE(AFT_ANALOG_LOGIC_CHANNELS);
				aft_analog_page.pfnDlgProc = AFT_Analog_PageDlgProc;
				
				aft_analog_page.lParam = (LPARAM) stuff;
				aft_analog_page.pfnCallback = AFT_Analog_PageCallbackProc;
				aft_analog_page.dwFlags = PSP_USECALLBACK;

				hpage = CreatePropertySheetPage(&aft_analog_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}
				break;

			//all ISDN BRI cards
			case AFT_ISDN_BRI_SHARK_SUBSYS_VENDOR:
			case A700_SHARK_SUBSYS_VENDOR:
				memset(&aft_bri_page, 0, sizeof(PROPSHEETPAGE));
				aft_bri_page.dwSize = sizeof(PROPSHEETPAGE);
				aft_bri_page.hInstance = hInst;
				aft_bri_page.pszTemplate = MAKEINTRESOURCE(AFT_BRI_LOGIC_CHANNELS);
				aft_bri_page.pfnDlgProc = AFT_BRI_PageDlgProc;
				
				aft_bri_page.lParam = (LPARAM) stuff;
				aft_bri_page.pfnCallback = AFT_BRI_PageCallbackProc;
				aft_bri_page.dwFlags = PSP_USECALLBACK;

				hpage = CreatePropertySheetPage(&aft_bri_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}
				break;

			//all Serial cards
			case AFT_2SERIAL_V35X21_SUBSYS_VENDOR:	//	0xA031
			case AFT_4SERIAL_V35X21_SUBSYS_VENDOR:	//	0xA032
			case AFT_2SERIAL_RS232_SUBSYS_VENDOR:	//	0xA033
			case AFT_4SERIAL_RS232_SUBSYS_VENDOR:	//	0xA034
				// logic channels page
				memset(&aft_serial_page, 0, sizeof(PROPSHEETPAGE));
				aft_serial_page.dwSize = sizeof(PROPSHEETPAGE);
				aft_serial_page.hInstance = hInst;
				aft_serial_page.pszTemplate = MAKEINTRESOURCE(AFT_SERIAL_CFG);
				aft_serial_page.pfnDlgProc = AFT_Serial_PageDlgProc;
				
				aft_serial_page.lParam = (LPARAM) stuff;
				aft_serial_page.pfnCallback = AFT_Serial_PageCallbackProc;
				aft_serial_page.dwFlags = PSP_USECALLBACK;

				// Create a property page and add it to the device manager's property sheet
				//
				//media page
				//
				//This page is added first so it will receive PSPCB_RELEASE (destroy)
				//notification last. So, there we will have to compare the old and new
				//configuration and restart "Virtual adapter" if cfg changed.
				//Also, there will be all the global clean-up.

				// logic channels page
				hpage = CreatePropertySheetPage(&aft_serial_page);
				if (!(*AddPage)(hpage, lParam)){
					DestroyPropertySheetPage(hpage);
				}

				break;

			default:
				MessageBox(GetActiveWindow(), "Invalid Card Sub type!", "Error",
					MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
				break;
			}
			break;

		case SDLA_ADSL:
			//ADSL_CFG
			memset(&s518_adsl_page, 0, sizeof(PROPSHEETPAGE));
			s518_adsl_page.dwSize = sizeof(PROPSHEETPAGE);
			s518_adsl_page.hInstance = hInst;
			s518_adsl_page.pszTemplate = MAKEINTRESOURCE(ADSL_CFG);
			s518_adsl_page.pfnDlgProc = ADSL_PageDlgProc;
			
			s518_adsl_page.lParam = (LPARAM) stuff;
			s518_adsl_page.pfnCallback = ADSL_PageCallbackProc;
			s518_adsl_page.dwFlags = PSP_USECALLBACK;

			//ADSL logic channels page
			hpage = CreatePropertySheetPage(&s518_adsl_page);
			if (!(*AddPage)(hpage, lParam)){
				DestroyPropertySheetPage(hpage);
			}
			break;

		default:
			MessageBox(GetActiveWindow(), "Invalid Card type!", "Error",
				MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
			break;
		}//switch(iCardType)

		//driver cfg page
		memset(&drv_page, 0, sizeof(PROPSHEETPAGE));
		drv_page.dwSize = sizeof(PROPSHEETPAGE);
		drv_page.hInstance = hInst;
		drv_page.pszTemplate = MAKEINTRESOURCE(DRV_CFG);
		drv_page.pfnDlgProc = DrvPageDlgProc;
				
		drv_page.lParam = (LPARAM) stuff;
		drv_page.pfnCallback = DrvPageCallbackProc;
		drv_page.dwFlags = PSP_USECALLBACK;

		//global driver configuration page
		hpage = CreatePropertySheetPage(&drv_page);
		if (!(*AddPage)(hpage, lParam)){
			DestroyPropertySheetPage(hpage);
		}
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////

DLGPROC_RETURN_TYPE WINAPI AFT_T1E1_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch(msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DBG_SAM_CLASS, "WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if (T1E1CfgOnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DBG_SAM_CLASS, "Invalid Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}

		break;

	case WM_INITDIALOG:
		{
			if (okay == TRUE)
			{
				//Configuration was already read from the registry.
				//Initialize the visual controls.
				MediaInit(hkey, hdlg);
			}

			if (!okay)
			{					// no SampleInfo
				ShowWindow(GetDlgItem(hdlg, IDC_INFOLABEL), SW_HIDE);
				ShowWindow(GetDlgItem(hdlg, IDB_MOREINFO), SW_HIDE);
			}					// no SampleInfo
		}
		break;
		
   	case WM_DESTROY:
		Debug(DBG_SAM_CLASS, "Samclass:WM_DESTROY");
		{
			LPSTR infofile = (LPSTR) GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile){
				GlobalFree((HGLOBAL) infofile);
			}
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_SAM_CLASS, "DBG_SAM_CLASS:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		MediaInit(hkey, hdlg);
		break;
		
    case WM_COMMAND:
		switch(LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(),"WanpipePnp.hlp",HELP_CONTEXT,IDH_LINE_CFG);
			return TRUE;

		case IDR_T1:
			if(IsDlgButtonChecked(hdlg, IDR_T1))
			{	
				Globals_New_Cfg.front_end_cfg.media = WAN_MEDIA_T1;
				//set T1 default values:
				Globals_New_Cfg.front_end_cfg.frame = WAN_FR_ESF;
				Globals_New_Cfg.front_end_cfg.lcode = WAN_LCODE_B8ZS;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = 1;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_clock = WAN_NORMAL_CLK;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_ref_clock = 0;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode = WANOPT_NO;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel = DEFAULT_TE_RX_SLEVEL;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm = WANOPT_NO;

				//Update visual controls with latest information stored in "Globals_New_Cfg".
				MediaInit(hkey, hdlg);
			}
			return TRUE;
		case IDR_E1:
			if(IsDlgButtonChecked(hdlg, IDR_E1))
			{	
				Globals_New_Cfg.front_end_cfg.media = WAN_MEDIA_E1;
				//set E1 default values:
				Globals_New_Cfg.front_end_cfg.frame = WAN_FR_CRC4;
				Globals_New_Cfg.front_end_cfg.lcode = WAN_LCODE_HDB3;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = 0;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.sig_mode = WAN_TE1_SIG_CCS;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_clock = WAN_NORMAL_CLK;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_ref_clock = 0;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode = WANOPT_NO;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel = DEFAULT_TE_RX_SLEVEL;
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm = WANOPT_NO;

				//Update visual controls with latest information stored in "Globals_New_Cfg".
				MediaInit(hkey, hdlg);
			}
			return TRUE;

		case COMBO_HIGH_IMPEDANCE_MODE:
			switch(HIWORD(wParam))
			{
			case CBN_SELCHANGE:
				int high_impedance_mode = SendDlgItemMessage(hdlg, COMBO_HIGH_IMPEDANCE_MODE, CB_GETCURSEL, 0, 0);
				switch(high_impedance_mode)
				{
				case WANOPT_YES:
					Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode = WANOPT_YES;
					break;
				default:
					Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode = WANOPT_NO;
				}

				init_te_rx_slevel_combobox(hdlg);
				break;
			}
			return TRUE;
		case COMBO_IGNORE_YELLOW_ALARM:
			switch(HIWORD(wParam))
			{
			case CBN_SELCHANGE:
				Globals_New_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm =
					cfg_combobox_get_value(hdlg, COMBO_IGNORE_YELLOW_ALARM, &ignore_yellow_alarm_mode[0]);
				break;
			}
			return TRUE;
		}//switch(LOWORD(wParam)) - process command notification
		break;
	}//switch(msg) - process message

	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

static void disable_enable_T1_controls(HWND hdlg, BOOL flag)
{
	EnableWindow( GetDlgItem(hdlg, COMBO_FRAMING_T1),			flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_LINE_DECODING_T1),		flag);
}

static void disable_enable_E1_controls(HWND hdlg, BOOL flag)
{
	EnableWindow( GetDlgItem(hdlg, COMBO_FRAMING_E1),			flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_LINE_DECODING_E1),		flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_SIGNALLING_E1),		flag);
}

static void disable_enable_T1E1_controls(HWND hdlg, BOOL flag)
{
	EnableWindow( GetDlgItem(hdlg, COMBO_DSU_CSU_CLK_MODE),		flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_CLK_REFERENCE_PORT),	flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_HIGH_IMPEDANCE_MODE),	flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_IGNORE_YELLOW_ALARM),	flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_TE_RX_SLEVEL),			flag);
	EnableWindow( GetDlgItem(hdlg, COMBO_LBO),					flag);
}

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK AFT_T1E1_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DRV_CFG_PAGE, "%s(): PSPCB_RELEASE()\n", __FUNCTION__);

		RegCloseKey(hkey);

		if(memcmp(&Globals_Old_Cfg.wandev_conf, &Globals_New_Cfg.wandev_conf, sizeof(wandev_conf_t))){
			int ind;
			unsigned char *old_wandev_conf = (unsigned char *)&Globals_Old_Cfg.wandev_conf;
			unsigned char *new_wandev_conf = (unsigned char *)&Globals_New_Cfg.wandev_conf;

			Debug(DRV_CFG_PAGE, "%s(): wandev_conf changed\n", __FUNCTION__);

			if(memcmp(&Globals_Old_Cfg.wandev_conf.fe_cfg, &Globals_New_Cfg.wandev_conf.fe_cfg, sizeof(sdla_fe_cfg_t))){
				Debug(DRV_CFG_PAGE, "%s(): fe_cfg changed\n", __FUNCTION__);
			}

			//print the two structures for comparison
			for(ind = 0; ind < sizeof(wandev_conf_t); ind++){
				if(old_wandev_conf[ind] != new_wandev_conf[ind]){
					Debug(DRV_CFG_PAGE, "%s(): old_wandev_conf[%d]: 0x%02X != new_wandev_conf[%d]: 0x%02X\n",
						__FUNCTION__, ind, old_wandev_conf[ind], ind, new_wandev_conf[ind]);					
				}
			}

			Debug(DRV_CFG_PAGE, "%s(): offsetof(wandev_conf_t, electrical_interface): %d\n", __FUNCTION__, offsetof(wandev_conf_t, electrical_interface));
			Debug(DRV_CFG_PAGE, "%s(): offsetof(wandev_conf_t, comm_port): %d\n", __FUNCTION__, offsetof(wandev_conf_t, comm_port));
			Debug(DRV_CFG_PAGE, "%s(): offsetof(wandev_conf_t, data_size): %d\n", __FUNCTION__, offsetof(wandev_conf_t, data_size));
			Debug(DRV_CFG_PAGE, "%s(): offsetof(wandev_conf_t, fe_cfg): %d\n", __FUNCTION__, offsetof(wandev_conf_t, fe_cfg));
			Debug(DRV_CFG_PAGE, "%s(): offsetof(wandev_conf_t, hwec_conf): %d\n", __FUNCTION__, offsetof(wandev_conf_t, hwec_conf));
/*
Thu Jul 23 12:25:03.780 2009 (GMT-4): AFT_T1E1_PageCallbackProc(): offsetof(wandev_conf_t, electrical_interface): 86
Thu Jul 23 12:25:03.842 2009 (GMT-4): AFT_T1E1_PageCallbackProc(): offsetof(wandev_conf_t, comm_port): 68
Thu Jul 23 12:25:03.889 2009 (GMT-4): AFT_T1E1_PageCallbackProc(): offsetof(wandev_conf_t, data_size): 144
Thu Jul 23 12:25:03.936 2009 (GMT-4): AFT_T1E1_PageCallbackProc(): offsetof(wandev_conf_t, fe_cfg): 612
Thu Jul 23 12:25:03.983 2009 (GMT-4): AFT_T1E1_PageCallbackProc(): offsetof(wandev_conf_t, hwec_conf): 720
*/
		}

		//compare the configuration structures, restart if different.
		if((released_by_OK_button == 1) &&
		   memcmp(&Globals_Old_Cfg, &Globals_New_Cfg, sizeof(Globals_New_Cfg)))
		{
			if(yes_no_question("Device Configuration was changed. Do you want the Device to be restarted now?")
					== IDYES){
				restart_device();
			}else{
				MessageBox(GetActiveWindow(), (LPCTSTR)"Device was NOT restarted.",
					"Note for the user", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
			}
		}
		delete (SETUPSTUFF*) p->lParam;
		return 0;//return code is ignored
	}
	return TRUE;	// matters only for PSPCB_CREATE
}


static void ReadConfigurationFromRegistry()
{
	int		i;
	char	ch_group_name[TMP_BUFFER_LEN];
	char	szTemp[TMP_BUFFER_LEN], szTemp1[TMP_BUFFER_LEN];
	DWORD	dwTemp;

	Debug(DBG_SAM_CLASS, "\n******** ReadConfigurationFromRegistry() *******");

	GetRegistryStringValue(hkey, "DeviceName", Globals_Old_Cfg.DeviceName, &dwTemp);

	//////////////////////////////////////////////////////////////////////
	//AutoWanpipeNumber and UserWanpipeNumber are related
	GetRegistryStringValue(hkey, "AutoWanpipeNumber", szTemp, &dwTemp);//read-only
	iAutoWanpipeNumber = atoi(szTemp);

	GetRegistryStringValue(hkey, "UserWanpipeNumber", szTemp, &dwTemp);//read/write
	Globals_Old_Cfg.wanpipe_number = atoi(szTemp);

	//////////////////////////////////////////////////////////////////////
	GetRegistryStringValue(hkey, "CardType", szTemp, &dwTemp);
	iCardType = atoi(szTemp);

	GetRegistryStringValue(hkey, "CardSubType", szTemp, &dwTemp);
	iCardSubType = atoi(szTemp);

	//////////////////////////////////////////////////////////////////////
	GetRegistryStringValue(hkey, "driver_mode", szTemp, &dwTemp);
	iDriverMode = atoi(szTemp);

	//////////////////////////////////////////////////////////////////////
	//HWEC options
	GetRegistryStringValue(hkey, "HWEC_CLKSRC", szTemp, &dwTemp);
	Globals_Old_Cfg.wandev_conf.hwec_conf.clk_src = atoi(szTemp);

	//////////////////////////////////////////////////////////////////////
	//TDMV options
	GetRegistryStringValue(hkey, "TDMV_DCHAN", Globals_Old_Cfg.dchan_active_ch_str, &dwTemp);

	//////////////////////////////////////////////////////////////////////
	//Media Type and other parameters which a valid for ALL media types
	Debug(DBG_SAM_CLASS, "\n******** Media Config *******");

	GetRegistryStringValue(hkey, "Media", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.media = atoi(szTemp);

	GetRegistryStringValue(hkey, "aft_front_end_line_number", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.line_no = atoi(szTemp);

	GetRegistryStringValue(hkey, "tdmv_law", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.tdmv_law = atoi(szTemp);

	GetRegistryStringValue(hkey, "ExternalSynchronization", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.network_sync = atoi(szTemp);

	//Framing
	GetRegistryStringValue(hkey, "Framing", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.frame = atoi(szTemp);

	//get the line decoding
	GetRegistryStringValue(hkey, "LDecoding", szTemp, &dwTemp);
	Globals_Old_Cfg.front_end_cfg.lcode = atoi(szTemp);

	//////////////////////////////////////////////////////////////////////
	//Serial card data.
	//
	//Serial Electrical interface (not used on A140, only used on S514)
	GetRegistryStringValue(hkey, "electrical_interface", szTemp, &dwTemp);
	Globals_Old_Cfg.wandev_conf.electrical_interface = atoi(szTemp);

	//Serial Clock Source
	GetRegistryStringValue(hkey, "clock_source", szTemp, &dwTemp);
	Globals_Old_Cfg.wandev_conf.clocking = atoi(szTemp);

	//BaudRate
	GetRegistryStringValue(hkey, "BaudRate", szTemp, &dwTemp);
	Globals_Old_Cfg.wandev_conf.bps = atoi(szTemp);//bps - bits per-second

	//serial_connection_type
	GetRegistryStringValue(hkey, "serial_connection_type", szTemp, &dwTemp);
	Globals_Old_Cfg.serial_connection_type = atoi(szTemp);

	//BaudRate
	GetRegistryStringValue(hkey, "serial_line_coding", szTemp, &dwTemp);
	Globals_Old_Cfg.serial_line_coding = atoi(szTemp);

	//BaudRate
	GetRegistryStringValue(hkey, "serial_line_idle", szTemp, &dwTemp);
	Globals_Old_Cfg.serial_line_idle = atoi(szTemp);
	///////////////// end of serial data /////////////////////////////////

	//parameters for each media type. note that storage is a union!
	switch(Globals_Old_Cfg.front_end_cfg.media)
	{
	case WAN_MEDIA_T1:
	case WAN_MEDIA_E1:
		//get the LBO
		GetRegistryStringValue(hkey, "LBO", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.lbo = atoi(szTemp);

		//get the clocking mode
		GetRegistryStringValue(hkey, "ClkMode", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.te_clock = atoi(szTemp);

		//get the "clock reference port"
		GetRegistryStringValue(hkey, "ClkRefPort", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.te_ref_clock = atoi(szTemp);

		//get "e1 signalling"
		GetRegistryStringValue(hkey, "E1Signalling", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.sig_mode = atoi(szTemp);

		//get "high_impedance_mode"
		GetRegistryStringValue(hkey, "HighImpedanceMode", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode = atoi(szTemp);

		//get Receiver Sensitivity (Max Cable Loss Allowed)
		GetRegistryStringValue(hkey, "TE_RX_SLEVEL", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel = atoi(szTemp);

		//get "TE_IGNORE_YEL"
		GetRegistryStringValue(hkey, "TE_IGNORE_YEL", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm = atoi(szTemp);
		break;

	case WAN_MEDIA_NONE:
	case WAN_MEDIA_56K:
		;//do nothing
		break;

	case WAN_MEDIA_BRI:
		//A500/B700 BRI
		GetRegistryStringValue(hkey, "aft_bri_clock_mode", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.bri.clock_mode = atoi(szTemp);

		snprintf(szTemp, sizeof(szTemp), "aft_bri_line_%d_module_type", Globals_Old_Cfg.front_end_cfg.line_no);
		GetRegistryStringValue(hkey, szTemp, Globals_Old_Cfg.BriModuleType, &dwTemp);
		break;

	case WAN_MEDIA_FXOFXS:
		//A200/A400/B700 Analog
		GetRegistryStringValue(hkey, "remora_fxo_operation_mode_name", Globals_Old_Cfg.front_end_cfg.cfg.remora.opermode_name, &dwTemp);

		GetRegistryStringValue(hkey, "RM_BATTTHRESH", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.remora.battthresh = atoi(szTemp);

		GetRegistryStringValue(hkey, "RM_BATTDEBOUNCE", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.remora.battdebounce = atoi(szTemp);

		GetRegistryStringValue(hkey, "RM_FXO_TAPPING", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.remora.rm_mode = atoi(szTemp);

		GetRegistryStringValue(hkey, "RM_FXO_TAPPING_OFF_HOOK_THRESHOLD", szTemp, &dwTemp);
		Globals_Old_Cfg.front_end_cfg.cfg.remora.ohthresh = atoi(szTemp);
		break;

	default:
		snprintf(szTemp, TMP_BUFFER_LEN, "Invalid Media type! (0x%X)", 
			Globals_Old_Cfg.front_end_cfg.media);
		MessageBox(GetActiveWindow(), szTemp, "Error",
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return;
	}

	//////////////////////////////////////////////////////////////
	//Driver Configuration
	Debug(DBG_SAM_CLASS, "\n******** Driver Config *******");

	//LogErrors
	GetRegistryStringValue(hkey, "LogErrors", szTemp, &dwTemp);
	Globals_Old_Cfg.log_errors = atoi(szTemp);

	//StartPortWhenComputerStartsFlag
	GetRegistryStringValue(hkey, "StartPortWhenComputerStartsFlag", szTemp, &dwTemp);
	Globals_Old_Cfg.start_port_when_computer_starts = atoi(szTemp);

	///////////////////////////////////////////////////////////////////
	//AFT
	Debug(DBG_SAM_CLASS, "*********  Reading AFT Configuration  ********\n");

	GetRegistryStringValue(hkey, "aft_number_of_logic_channels", szTemp, &dwTemp);
	Globals_Old_Cfg.aft_number_logic_channels = atoi(szTemp);
	
	for(i=0; i < Globals_Old_Cfg.aft_number_logic_channels; i++)
	{	//get active channels
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_active_ch", i);
		GetRegistryStringValue(hkey, ch_group_name,
			(char*)Globals_Old_Cfg.aft_logic_ch_cfg[i].active_ch_str, &dwTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.active_ch_str[%d]: %s\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].active_ch_str);

		//get line mode
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_line_mode", i);
		GetRegistryStringValue(hkey, ch_group_name,
			(char*)Globals_Old_Cfg.aft_logic_ch_cfg[i].line_mode, &dwTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.line_mode[%d]: %s\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].line_mode);

		//get MTU
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_mtu", i);
		GetRegistryStringValue(hkey, ch_group_name,
			(char*)Globals_Old_Cfg.aft_logic_ch_cfg[i].mtu, &dwTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.mode[%d]: %s\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].mtu);

		//get operational mode
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_operational_mode", i);
		GetRegistryStringValue(hkey, ch_group_name,
			(char*)Globals_Old_Cfg.aft_logic_ch_cfg[i].operational_mode, &dwTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.operational_mode[%d]: %s\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].operational_mode);

		//get Idle char
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_idle_char", i);
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.aft_logic_ch_cfg[i].decimal_idle_char = atoi(szTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.aft_logic_ch_cfg[%d].decimal_idle_char: 0x%02X\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].decimal_idle_char);

		//get Analog Module Type. It's a read only inforamation, no need to write it back.
		snprintf(ch_group_name, TMP_BUFFER_LEN, "aft_logic_channel_%d_analog_module_type", i);
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.aft_logic_ch_cfg[i].analog_module_type = atoi(szTemp);

		Debug(DBG_SAM_CLASS, "Globals_Old_Cfg.aft_logic_ch_cfg[%d].analog_module_type: 0x%02X\n",
			i, Globals_Old_Cfg.aft_logic_ch_cfg[i].analog_module_type);
	}

	Debug(DBG_SAM_CLASS, "*********  Finished Reading AFT Configuration  ********\n");

	///////////////////////////////////////////////////////////////////
	//ADSL
	if(iCardType == SDLA_ADSL){
		Debug(DBG_SAM_CLASS, "*********  Reading ADSL Configuration  ********\n");
		//vpi
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslVpi");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.Vpi = atoi(szTemp);
		
		//vci
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslVci");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.Vci = atoi(szTemp);

		//rfc
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslEncapMode");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.EncapMode = atoi(szTemp);

		//Standard
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslStandard");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.Standard = atoi(szTemp);

		//rx cell count
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslRxCellCount");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.RxCellCount = atoi(szTemp);

		//???????????????????????
		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslTxPowerAtten");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.TxPowerAtten = atoi(szTemp);

		snprintf(ch_group_name, TMP_BUFFER_LEN, "AdslCodingGain");
		GetRegistryStringValue(hkey, ch_group_name, szTemp, &dwTemp);
		Globals_Old_Cfg.adsl_cfg.CodingGain = atoi(szTemp);
		//????????????????????????

		Debug(DBG_SAM_CLASS, "*********  Finished Reading ADSL Configuration  ********\n");
	}
	/////////////////////////////////////////////////////////////////////////////////
}

static BOOL SaveT1E1CfgData(HWND hdlg)
{	
	char local_tmp_buff[TMP_BUFFER_LEN];

	Debug(DBG_SAM_CLASS, "%s()", __FUNCTION__);
	
	//
	//saving Media Type, NOT for Analog card!
	//
	if(Globals_Old_Cfg.front_end_cfg.media != WAN_MEDIA_FXOFXS){

		if(IsDlgButtonChecked(hdlg, IDR_T1)){
			Globals_New_Cfg.front_end_cfg.media = WAN_MEDIA_T1;
			SetRegistryStringValue(hkey, "Media", WAN_MEDIA_T1);
		}else if(IsDlgButtonChecked(hdlg, IDR_E1)){
			Globals_New_Cfg.front_end_cfg.media = WAN_MEDIA_E1;
			SetRegistryStringValue(hkey, "Media", WAN_MEDIA_E1);
		}
	}

	//
	//Framing
	//
	int fr;
	if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
	{	
		fr = SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_GETCURSEL, 0, 0);
		fr++;//index is zerobased
		SetRegistryStringValue(hkey, "Framing", fr);

		if(fr == 1)
		{	Globals_New_Cfg.front_end_cfg.frame = WAN_FR_ESF;
		}
		else if(fr == 2)
		{	Globals_New_Cfg.front_end_cfg.frame = WAN_FR_D4;
		}
		else if(fr == 3)
		{	Globals_New_Cfg.front_end_cfg.frame = WAN_FR_ESF_JAPAN;
		}
	}
	else if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{	fr = SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_GETCURSEL, 0, 0);
		switch(fr)
		{
		case 0:
			fr = WAN_FR_CRC4;
			break;
		case 1:
			fr = WAN_FR_NCRC4;
			break;
		case 2:
			fr = WAN_FR_UNFRAMED;
			break;
		default:
			fr = WAN_FR_CRC4;
			break;
		}

		SetRegistryStringValue(hkey, "Framing", fr);
		Globals_New_Cfg.front_end_cfg.frame = fr;
	}else
	{	//use the old value
		Globals_New_Cfg.front_end_cfg.frame = Globals_Old_Cfg.front_end_cfg.frame;
	}
	Debug(DBG_SAM_CLASS, "SaveCfgData():Framing : %d", fr);

	//
	//LBO
	//
	int lbo = SendDlgItemMessage(hdlg, COMBO_LBO, CB_GETCURSEL, 0, 0);
	if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
	{	
		lbo++;//index is zerobased
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = lbo;
	}else if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1){	
		switch(lbo)
		{
		case 0:
			Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = WAN_E1_120;
			break;
		case 1:
			Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = WAN_E1_75;
			break;
		default:
			Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo = WAN_E1_120;
			break;
		}
	}
	SetRegistryStringValue(hkey, "LBO", Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo);

	//
	//Line decoding
	//
	int ldc;

	switch(Globals_New_Cfg.front_end_cfg.media)
	{
	case WAN_MEDIA_T1:
	case WAN_MEDIA_E1:
		if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
		{
			ldc = SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_GETCURSEL, 0, 0);
			SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_GETLBTEXT, ldc, (LPARAM)local_tmp_buff);
		}else{
			ldc = SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_GETCURSEL, 0, 0);
			SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_GETLBTEXT, ldc, (LPARAM)local_tmp_buff);
		}

		if(strncmp(local_tmp_buff, AMI_STR, TMP_BUFFER_LEN) == 0){
			ldc = WAN_LCODE_AMI;
		}else if(strncmp(local_tmp_buff, B8ZS_STR, TMP_BUFFER_LEN) == 0){
			ldc = WAN_LCODE_B8ZS;
		}else{
			ldc = WAN_LCODE_HDB3;
		}

		SetRegistryStringValue(hkey, "LDecoding", ldc);
		Debug(DBG_SAM_CLASS, "SaveCfgData():ldc : %d", ldc);
		Globals_New_Cfg.front_end_cfg.lcode = ldc;
		break;

	default:
		//use the old value
		Globals_New_Cfg.front_end_cfg.lcode = Globals_Old_Cfg.front_end_cfg.lcode;
		break;
	}

	//
	//DSU/CSU Clocking Mode
	//
	if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1 || 
		Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{	int clck_mode = SendDlgItemMessage(hdlg, COMBO_DSU_CSU_CLK_MODE, CB_GETCURSEL, 0, 0);
		clck_mode++;
		SetRegistryStringValue(hkey, "ClkMode", clck_mode);

		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_clock = clck_mode;

		//the "clock_ref_port" is just an index in "COMBO_CLK_REFERENCE_PORT"
		int clock_ref_port = SendDlgItemMessage(hdlg, COMBO_CLK_REFERENCE_PORT, CB_GETCURSEL, 0, 0);
		SetRegistryStringValue(hkey, "ClkRefPort", clock_ref_port);

		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_ref_clock = clock_ref_port;
	}

	//
	//E1 Signalling
	//
	if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{	int signalling = SendDlgItemMessage(hdlg, COMBO_SIGNALLING_E1, CB_GETCURSEL, 0, 0);

		switch(signalling)
		{
		case 0:
			signalling = WAN_TE1_SIG_CCS;
			break;
		case 1:
			signalling = WAN_TE1_SIG_CAS;
			break;
		default:
			signalling = WAN_TE1_SIG_CCS;
			break;
		}
		SetRegistryStringValue(hkey, "E1Signalling", signalling);
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.sig_mode = signalling;
	}

	//
	//High Impedance Mode
	//
	SetRegistryStringValue(hkey, "HighImpedanceMode", 
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode);

	//
	//Monitoring Gaing for the High Impedance Mode
	//
	Globals_New_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel = get_te_rx_slevel_value(hdlg);
	
	SetRegistryStringValue(hkey, "TE_RX_SLEVEL", 
			Globals_New_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel);

	//
	//Ignore Yellow Alarm
	//
	SetRegistryStringValue(hkey, "TE_IGNORE_YEL", 
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm);

	return TRUE;
}


static void MediaInit(HKEY	hkey, HWND hdlg)
{
	int n, ind;

	//
	//Media Type. Based on Media Type disable or enable relevant controls.
	//
	switch(Globals_New_Cfg.front_end_cfg.media)
	{
	case WAN_MEDIA_T1:
		CheckDlgButton(hdlg, IDR_T1, BST_CHECKED);

		disable_enable_T1_controls(hdlg,	TRUE);
		disable_enable_E1_controls(hdlg,	FALSE);
		disable_enable_T1E1_controls(hdlg,	TRUE);
		break;
	case WAN_MEDIA_E1:
		CheckDlgButton(hdlg, IDR_E1, BST_CHECKED);

		disable_enable_T1_controls(hdlg,	FALSE);
		disable_enable_E1_controls(hdlg,	TRUE);
		disable_enable_T1E1_controls(hdlg,	TRUE);
		break;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////

	//
	//Framing
	//
	//T1
	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_RESETCONTENT, 0, 0);
	n = 0;
	while(framing_options_T1[n] != NULL)
	{	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_ADDSTRING, 0, (LPARAM)framing_options_T1[n]);
		n++;
	}

	//choose ESF as a default in T1 for now
	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_SETCURSEL, 0, 0);

	//E1
	SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_RESETCONTENT, 0, 0);
	n = 0;
	while(framing_options_E1[n] != NULL)
	{	SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_ADDSTRING, 0, (LPARAM)framing_options_E1[n]);
		n++;
	}
	//choose CRC4 as a default in E1 for now
	SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_SETCURSEL, 0, 0);

	//get the framing
	if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
	{	
		//set focus to the right line
		if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_ESF)
		{	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_SETCURSEL, 0, 0);
		}
		else if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_D4)
		{	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_SETCURSEL, 1, 0);
		}
		else if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_ESF_JAPAN)
		{	SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_SETCURSEL, 2, 0);
		}
	}
	else if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{	//set focus to the right line
		if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_CRC4)
		{	SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_SETCURSEL, 0, 0);
		}else if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_NCRC4){
			SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_SETCURSEL, 1, 0);
		}else if(Globals_New_Cfg.front_end_cfg.frame == WAN_FR_UNFRAMED){
			SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_SETCURSEL, 2, 0);
		}
	}else
	{	
		SendDlgItemMessage(hdlg, COMBO_FRAMING_T1, CB_SETCURSEL, 0, 0);
		SendDlgItemMessage(hdlg, COMBO_FRAMING_E1, CB_SETCURSEL, 0, 0);
	}

	//////////////////////////////////////////////////////////////////////////////////////
	//T1 LBO
	//////////////////////////////////////////////////////////////////////////////////////
	SendDlgItemMessage(hdlg, COMBO_LBO, CB_RESETCONTENT, 0, 0);

	if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
	{	
		n = 0;
		while(t1_lbo_options[n] != NULL)
		{	SendDlgItemMessage(hdlg, COMBO_LBO, CB_ADDSTRING, 0, (LPARAM)t1_lbo_options[n]);
			n++;
		}
		//select default lbo
		SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, 0, 0);

		switch(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo)
		{
		case WAN_T1_LBO_0_DB:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_LBO_0_DB - 1), 0);
			break;
		case WAN_T1_LBO_75_DB:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_LBO_75_DB - 1), 0);
			break;
		case WAN_T1_LBO_15_DB:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_LBO_15_DB - 1), 0);
			break;
		case WAN_T1_LBO_225_DB:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_LBO_225_DB - 1), 0);
			break;
		case WAN_T1_0_110:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_0_110 - 1), 0);
			break;
		case WAN_T1_110_220:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_110_220 - 1), 0);
			break;
		case WAN_T1_220_330:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_220_330 - 1), 0);
			break;
		case WAN_T1_330_440:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_330_440 - 1), 0);
			break;
		case WAN_T1_440_550:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_440_550 - 1), 0);
			break;
		case WAN_T1_550_660:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, (WAN_T1_550_660 - 1), 0);
			break;
		}//switch(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo)
	}else if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{
		n = 0;
		while(e1_lbo_options[n] != NULL)
		{	SendDlgItemMessage(hdlg, COMBO_LBO, CB_ADDSTRING, 0, (LPARAM)e1_lbo_options[n]);
			n++;
		}
		//select default lbo
		SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, 0, 0);

		switch(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.lbo)
		{
		case WAN_E1_120:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, 0, 0);
			break;
		case WAN_E1_75:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, 1, 0);
			break;
		default:
			SendDlgItemMessage(hdlg, COMBO_LBO, CB_SETCURSEL, 0, 0);
			break;
		}
	}

	//
	//Line Decoding
	//
	//T1
	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_RESETCONTENT, 0, 0);
	for(n=0; n < 2; n++)
	{	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_ADDSTRING, 0, (LPARAM)line_decoding_T1[n]);
	}
	//select the default for now
	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_SETCURSEL, (WAN_LCODE_AMI - 1), 0);

	//E1
	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_RESETCONTENT, 0, 0);
	for(n=0; n < 2; n++)
	{	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_ADDSTRING, 0, (LPARAM)line_decoding_E1[n]);
	}
	//select the default for now
	SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_SETCURSEL, (WAN_LCODE_AMI - 1), 0);

	//get the line decoding
	if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1)
	{
		switch(Globals_New_Cfg.front_end_cfg.lcode)
		{
		case WAN_LCODE_B8ZS:
			SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_T1, CB_SETCURSEL, (WAN_LCODE_B8ZS - 1), 0);
			break;
		}
	}else if(Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{
		switch(Globals_New_Cfg.front_end_cfg.lcode)
		{
		case WAN_LCODE_HDB3:
			SendDlgItemMessage(hdlg, COMBO_LINE_DECODING_E1, CB_SETCURSEL, 1, 0);
			break;
		}
	}

	//
	//DSU/CSU clocking mode
	//
	SendDlgItemMessage(hdlg, COMBO_DSU_CSU_CLK_MODE, CB_RESETCONTENT, 0, 0);
	for(n=0; n < 2; n++){
		SendDlgItemMessage(hdlg, COMBO_DSU_CSU_CLK_MODE, CB_ADDSTRING, 0, (LPARAM)clocking_mode [n]);
	}
	//select the default for now
	SendDlgItemMessage(hdlg, COMBO_DSU_CSU_CLK_MODE, CB_SETCURSEL, (WAN_NORMAL_CLK - 1), 0);


	//get the clocking mode
	if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1 || 
		Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{
		SendDlgItemMessage(	hdlg, COMBO_DSU_CSU_CLK_MODE, CB_SETCURSEL, 
							(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_clock - 1), 0);
	}
	
	//
	//clock reference port
	//
	int max_ref_clock_port=0;
	switch(iCardType)
	{
	case SDLA_AFT:
		switch(iCardSubType)
		{
		case A101_1TE1_SUBSYS_VENDOR:
		case AFT_1TE1_SHARK_SUBSYS_VENDOR:
			max_ref_clock_port=0;//only "Not used" will be available
			break;

		case A101_2TE1_SUBSYS_VENDOR:
		case AFT_2TE1_SHARK_SUBSYS_VENDOR:
			max_ref_clock_port=2;
			break;

		case A104_4TE1_SUBSYS_VENDOR:
		case AFT_4TE1_SHARK_SUBSYS_VENDOR:
			max_ref_clock_port=4;
			break;

		case AFT_8TE1_SHARK_SUBSYS_VENDOR:
			max_ref_clock_port=8;
			break;
		}
	}

	SendDlgItemMessage(hdlg, COMBO_CLK_REFERENCE_PORT, CB_RESETCONTENT, 0, 0);
	for(n=0; n <= max_ref_clock_port; n++){
		SendDlgItemMessage(hdlg, COMBO_CLK_REFERENCE_PORT, CB_ADDSTRING, 0, (LPARAM)clock_reference_port[n]);
	}

	//select the default ("Not Used") for now
	SendDlgItemMessage(hdlg, COMBO_CLK_REFERENCE_PORT, CB_SETCURSEL, 0, 0);

	//get the "clock reference port"
	if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1 || 
		Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{
		SendDlgItemMessage(	hdlg, COMBO_CLK_REFERENCE_PORT, CB_SETCURSEL, 
			(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.te_ref_clock), 0);
	}

	//
	//E1 Signalling
	//
	SendDlgItemMessage(hdlg, COMBO_SIGNALLING_E1, CB_RESETCONTENT, 0, 0);
	n=0;
	while(e1_signalling[n] != NULL)
	{	SendDlgItemMessage(hdlg, COMBO_SIGNALLING_E1, CB_ADDSTRING, 0, (LPARAM)e1_signalling[n]);
		n++;
	}

	//get the current E1 signalling
	if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_E1)
	{
		switch(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.sig_mode)
		{
		case WAN_TE1_SIG_CCS:
			SendDlgItemMessage(	hdlg, COMBO_SIGNALLING_E1, CB_SETCURSEL, 0, 0);
			break;
		case WAN_TE1_SIG_CAS:
			SendDlgItemMessage(	hdlg, COMBO_SIGNALLING_E1, CB_SETCURSEL, 1, 0);
			break;
		default:
			SendDlgItemMessage(	hdlg, COMBO_SIGNALLING_E1, CB_SETCURSEL, 0, 0);
			break;
		}
	}

	/////////////////////////////////////////////////////////////////////////////
	//High Impedance Mode
	SendDlgItemMessage(hdlg, COMBO_HIGH_IMPEDANCE_MODE, CB_RESETCONTENT, 0, 0);
	n=0;
	while(high_impedance_modes[n] != NULL)
	{	SendDlgItemMessage(	hdlg, COMBO_HIGH_IMPEDANCE_MODE, CB_ADDSTRING, 0, 
							(LPARAM)high_impedance_modes[n]);
		n++;
	}

	//select the current impedance mode
	SendDlgItemMessage(	hdlg, COMBO_HIGH_IMPEDANCE_MODE, CB_SETCURSEL, 
						Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode, 0);

	//select Receiver Sensitivity (Max Cable Loss Allowed)
	init_te_rx_slevel_combobox(hdlg);

	/////////////////////////////////////////////////////////////////////////////
	//Init Ignore Yellow Alarm combo box
	cfg_combobox_init(hdlg, COMBO_IGNORE_YELLOW_ALARM, &ignore_yellow_alarm_mode[0], 
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.ignore_yel_alarm);
	/////////////////////////////////////////////////////////////////////////////
}

void init_te_rx_slevel_combobox(HWND hdlg)
{
	const look_up_t *te_rx_slevel_ptr;

	if(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode){
		te_rx_slevel_ptr = &high_impedance_rx_slevel[0];
	}else{
		if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1){
			te_rx_slevel_ptr = &t1_normal_mode_rx_slevel[0];
		}else{
			te_rx_slevel_ptr = &e1_normal_mode_rx_slevel[0];
		}
	}

	cfg_combobox_init(hdlg, COMBO_TE_RX_SLEVEL, te_rx_slevel_ptr, 
		Globals_New_Cfg.front_end_cfg.cfg.te_cfg.rx_slevel);
}

int get_te_rx_slevel_value(HWND hdlg)
{
	const look_up_t *te_rx_slevel_ptr;

	if(Globals_New_Cfg.front_end_cfg.cfg.te_cfg.high_impedance_mode){
		te_rx_slevel_ptr = &high_impedance_rx_slevel[0];
	}else{
		if(	Globals_New_Cfg.front_end_cfg.media == WAN_MEDIA_T1){
			te_rx_slevel_ptr = &t1_normal_mode_rx_slevel[0];
		}else{
			te_rx_slevel_ptr = &e1_normal_mode_rx_slevel[0];
		}
	}

	return cfg_combobox_get_value(hdlg, COMBO_TE_RX_SLEVEL, te_rx_slevel_ptr);
}

/////////////////////////////////////////////////////////////////////////////////
//
// Functions to read/write values from/to Registry
//
DWORD GetRegistryDWordValue(HKEY hKey, LPTSTR szKeyname)
{
	union	{ DWORD d; BYTE db[4]; } data;  // will hold the data
	DWORD	dwSize = 4;

	data.d = 0;
	//reading twice to set dwTemp to needed value
	RegQueryValueEx(hKey, szKeyname, NULL, NULL, &(data.db[0]), &dwSize);
	RegQueryValueEx(hKey, szKeyname, NULL, NULL, &(data.db[0]), &dwSize);

	Debug(DBG_SAM_CLASS, "GetRegistryDWordValue():  %s: %d (0x%X)", 
		szKeyname, data.d, data.d);

	return(data.d);
}

void SetRegistryDWordValue(HKEY hKey, LPTSTR szKeyname, DWORD dwValue)
{
	union	{ DWORD d; BYTE db[4]; } data;  // will hold the data
	DWORD	dwSize = 4;

	Debug(DBG_SAM_CLASS, "SetRegistryDWordValue():  %s: %d (0x%X)", 
		szKeyname, dwValue, dwValue);

	dwSize = 4;
	data.d = dwValue;

	RegSetValueEx(hKey, szKeyname, 0, REG_DWORD, &(data.db[0]), dwSize);
}

void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize)
{	
	//reading twice to set dwTemp to needed value
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);

	//If debugging enabled, this will print out the name and the
	//value that was read.
	Debug(DBG_SAM_CLASS, "GetRegistryStringValue():  %s: %s", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue)
{
	DWORD	dwSize;
	
	dwSize = strlen(szValue)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szValue, dwSize);

	//If debugging enabled, this will print out the name and the
	//value that has been saved.
	Debug(DBG_SAM_CLASS, "SetRegistryStringValue(1):   %s: %s", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue)
{
	DWORD	dwSize;
	static char	szTemp[TMP_BUFFER_LEN];

	snprintf(szTemp, TMP_BUFFER_LEN, "%u", iValue);
	dwSize = strlen(szTemp)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szTemp, dwSize);

	//If debugging enabled, this will print out the name and the
	//value that has been saved.
	Debug(DBG_SAM_CLASS, "SetRegistryStringValue(2):   %s: %s", szKeyname, szTemp);
}

/////////////////////////////////////////////////////////////////////////////////

BOOL CheckActiveChannels(PUCHAR ActiveChannels, unsigned char media_type)
{
	if(parse_active_channel((PCHAR)ActiveChannels, media_type, (Globals_New_Cfg.front_end_cfg.frame == WAN_FR_UNFRAMED)) == 0){
		return FALSE;
	}else{
		return TRUE;
	}
}

/////////////////////////////////////////////////////////////////////////////////

/*============================================================================
 * TE1
 * Parse active channel string.
 *
 * Return ULONG value, that include 1 in position `i` if channels i is active.
 */
unsigned long parse_active_channel(char* val, unsigned char media_type, unsigned char is_unframed_e1)
{
#define SINGLE_CHANNEL	0x2
#define RANGE_CHANNEL	  0x1
	int channel_flag = 0;
	char* ptr = val;
	int channel = 0, start_channel = 0;
	unsigned long tmp = 0;
	
	Debug(DBG_SAM_CLASS, 
		"parse_active_channel(): input: %s, media_type: %d\n", val, media_type);
	
	if (strcmp(val,"ALL") == 0){
		return ENABLE_ALL_CHANNELS;
	}
	
	if(active_channels_str_invalid_characters_check(val) == NO){
		return 0;
	}
	
	while(*ptr != '\0') {
		if (isdigit(*ptr)) {
			channel = strtoul(ptr, &ptr, 10);
			channel_flag |= SINGLE_CHANNEL;
		} else {
			if (*ptr == '-') {
				channel_flag |= RANGE_CHANNEL;
				start_channel = channel;
			} else {
				tmp |= get_active_channels(channel_flag, start_channel, channel, media_type);
				channel_flag = 0;
			}
			ptr++;
		}
	}
	if (channel_flag){
		tmp |= get_active_channels(channel_flag, start_channel, channel, media_type);
	}
	
	if(check_channels(channel_flag, start_channel, channel, media_type, is_unframed_e1) == NO){
		tmp = 0;
	}
	return tmp;
}

int active_channels_str_invalid_characters_check(char* active_ch_str)
{
	char* ptr = active_ch_str;
	
	while(*ptr != '\0') {
		if (isdigit(*ptr)) {
			;//ok
		} else {
			if(*ptr == '-' || *ptr == '.') {
				;//ok
			}else{
				//invalid char
				printf("Found invalid character '%c' in active channels string!!", *ptr);
				return NO;
			}
		}
		ptr++;
	}//while()
	return YES;
}

/*============================================================================
* TE1
*/
unsigned long get_active_channels(int channel_flag, int start_channel,
                                  int stop_channel,  unsigned char media_type)
{
	int i = 0;
	unsigned long tmp = 0, mask = 0;
	
	if ((channel_flag & (SINGLE_CHANNEL | RANGE_CHANNEL)) == 0)
		return tmp;
	if (channel_flag & RANGE_CHANNEL) { /* Range of channels */
		for(i = start_channel; i <= stop_channel; i++) {
			mask = 1 << (i - 1);
			tmp |=mask;
		}
	} else { /* Single channel */
		mask = 1 << (stop_channel - 1);
		tmp |= mask;
	}
	return tmp;
}

int check_channels(int channel_flag, unsigned int start_channel,
                   unsigned int stop_channel, unsigned char media_type, unsigned char is_unframed_e1)
{
	Debug(DBG_SAM_CLASS, "check_channels(): media_type: %d, start_channel: %d, stop_channel: %d\n",
		media_type, start_channel, stop_channel);
	
	if ((channel_flag & 0x03) == 0)
		return NO;
	if ((channel_flag & RANGE_CHANNEL) && (start_channel > stop_channel))
		return NO;
	if (!(channel_flag & RANGE_CHANNEL))
		start_channel = stop_channel;
	if ( media_type == WAN_MEDIA_T1){
		
		if( (start_channel < 1) || (stop_channel > NUM_OF_T1_CHANNELS) )
			return NO;
	}else if(media_type == WAN_MEDIA_E1){
		
		if(is_unframed_e1){
			if( (start_channel < 1) || (stop_channel > NUM_OF_E1_TIMESLOTS + 1) ){
				return NO;
			}
		}else{
			if( (start_channel < 1) || (stop_channel > NUM_OF_E1_TIMESLOTS) ){
				return NO;
			}
		}

	}else if(media_type == WAN_MEDIA_BRI){
		if( (start_channel < 1) || (stop_channel > BRI_DCHAN_ACTIVE_CFG_CHAN) )
			return NO;
	}

	return YES;
}

int T1E1CfgOnNotify(
			HWND    ParentHwnd,
			LPNMHDR NmHdr,
			SETUPSTUFF * Params
		    )
{
    SP_DEVINSTALL_PARAMS	spDevInstall = {0};
	BOOL					configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = SaveT1E1CfgData(ParentHwnd);

		Debug(DBG_SAM_CLASS, "T1E1CfgOnNotify(): PSN_APPLY");
		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE){
			return HANDLE_OK;
		}else{
			return HANDLE_ERROR;
		}

	case PSN_QUERYCANCEL:
		Debug(DBG_SAM_CLASS, "T1E1CfgOnNotify() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
    default:
		Debug(DBG_SAM_CLASS, "T1E1CfgOnNotify() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
} 

